// Adafruit.io
#include <WiFi.h>
#include <Adafruit_MQTT.h>
#include <Adafruit_MQTT_Client.h>

#define AIO_SERVER      "io.adafruit.com"
#define AIO_SERVERPORT  1883
#define AIO_USERNAME    "XXXXXXX"
#define AIO_KEY         "XXXXXXX"
WiFiClient client;

Adafruit_MQTT_Client mqtt(&client, AIO_SERVER, AIO_SERVERPORT, AIO_USERNAME, AIO_USERNAME, AIO_KEY);
Adafruit_MQTT_Publish CO2 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/CO2");
Adafruit_MQTT_Publish PM = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/PM");
Adafruit_MQTT_Publish Humidity = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/Humidity");
Adafruit_MQTT_Publish Temperature = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/Temperature");

const char* ssid = "XXXXXX";
const char* password = "XXXXXXX";

// PMS5003
#include <SoftwareSerial.h>
SoftwareSerial pmsSerial(17, 16);

// MHZ19
#include "MHZ19.h"

const int pwmpin = 14;

MHZ19 *mhz19_pwm = new MHZ19(pwmpin);

// DHT11
#include "DHT.h"

#define DHTPIN 32
#define DHTTYPE DHT11

DHT dht(DHTPIN, DHTTYPE);

int i = 0;

// BUZZER
int buzzerPin = 5;

struct pms5003data {
  uint16_t framelen;
  uint16_t pm10_standard, pm25_standard, pm100_standard;
  uint16_t pm10_env, pm25_env, pm100_env;
  uint16_t particles_03um, particles_05um, particles_10um, particles_25um, particles_50um, particles_100um;
  uint16_t unused;
  uint16_t checksum;
};

struct pms5003data data;

boolean readPMSdata(Stream *s) {
  if (!s->available()) {
    return false;
  }

  // Read a byte at a time until we get to the special '0x42' start-byte
  if (s->peek() != 0x42) {
    s->read();
    return false;
  }

  // Now read all 32 bytes
  if (s->available() < 32) {
    return false;
  }

  uint8_t buffer[32];
  uint16_t sum = 0;
  s->readBytes(buffer, 32);

  // get checksum ready
  for (uint8_t i = 0; i < 30; i++) {
    sum += buffer[i];
  }

  /* debugging
     for (uint8_t i=2; i<32; i++) {
       Serial.print("0x"); Serial.print(buffer[i], HEX); Serial.print(", ");
     }
     Serial.println();
  */

  // The data comes in endian'd, this solves it so it works on all platforms
  uint16_t buffer_u16[15];
  for (uint8_t i = 0; i < 15; i++) {
    buffer_u16[i] = buffer[2 + i * 2 + 1];
    buffer_u16[i] += (buffer[2 + i * 2] << 8);
  }

  // put it into a nice struct :)
  memcpy((void *)&data, (void *)buffer_u16, 30);

  if (sum != data.checksum) {
    // Serial.println("Checksum failure");
    return false;
  }
  // success!
  return true;
}

void setup() {
  // our debugging output
  Serial.begin(9600);

  WiFi.begin(ssid, password);

  // sensor baud rate is 9600
  pmsSerial.begin(9600);
  // Serial.print("MH-Z19 now warming up...  status:");

  pinMode(buzzerPin, OUTPUT);
  
  dht.begin();
}

void loop() {
  // tone(buzzerPin, 1000);
  // delay(1000);
  // noTone(buzzerPin);
  // delay(1000);

  if (!mqtt.ping(3)) {
    // reconnect to adafruit
    if (!mqtt.connected()) {
      mqtt.connect();
    }
  }

  if (readPMSdata(&pmsSerial)) {
    // reading data was successful!
    // Serial.println();
    // Serial.println("---------------------------------------");
    // Serial.println("Concentration Units (standard)");
    // Serial.print("PM 1.0: "); Serial.print(data.pm10_standard);
    // Serial.print("\t\tPM 2.5: "); Serial.print(data.pm25_standard);
    // Serial.print("\t\tPM 10: "); Serial.println(data.pm100_standard);
    // Serial.println("---------------------------------------");
    // Serial.println("Concentration Units (environmental)");
    // Serial.print("PM 1.0: "); Serial.print(data.pm10_env);
    Serial.print("n5.val=");
    Serial.print(data.pm10_env);
    Serial.write(0xff);
    Serial.write(0xff);
    Serial.write(0xff);  
    // Serial.print("\t\tPM 2.5: "); Serial.print(data.pm25_env);
    Serial.print("n0.val=");
    Serial.print(data.pm25_env);
    Serial.write(0xff);
    Serial.write(0xff);
    Serial.write(0xff);    
    // Serial.print("\t\tPM 10: "); Serial.println(data.pm100_env);
    Serial.print("n7.val=");
    Serial.print(data.pm100_env);
    Serial.write(0xff);
    Serial.write(0xff);
    Serial.write(0xff);     
    // Serial.println("---------------------------------------");
    // Serial.print("Particles > 0.3um / 0.1L air:"); Serial.println(data.particles_03um);
    // Serial.print("Particles > 0.5um / 0.1L air:"); Serial.println(data.particles_05um);
    // Serial.print("Particles > 1.0um / 0.1L air:"); Serial.println(data.particles_10um);
    // Serial.print("Particles > 2.5um / 0.1L air:"); Serial.println(data.particles_25um);
    // Serial.print("Particles > 5.0um / 0.1L air:"); Serial.println(data.particles_50um);
    // Serial.print("Particles > 10.0 um / 0.1L air:"); Serial.println(data.particles_100um);
    // Serial.println("---------------------------------------");
    int co2ppm = mhz19_pwm->getPpmPwm();
    // Serial.print("co2: ");
    Serial.print("n3.val=");
    Serial.print(co2ppm);
    Serial.write(0xff);
    Serial.write(0xff);
    Serial.write(0xff);
    if (co2ppm >= 700) {     // vanaf 1500 ppm wordt indoor 
      tone(buzzerPin, 1000);  // het concentratie vermogen aangetast.
      delay(1000);
      noTone(buzzerPin);
      delay(1000);    
    }
    // Serial.println("---------------------------------------");
    // Serial.print(
    
    // Reading temperature or humidity takes about 250 milliseconds!
    // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
    float h = dht.readHumidity();
    int h1 = h;
    // Read temperature as Celsius (the default)
    float t = dht.readTemperature();
    int t1 = t * 10;
    // Read temperature as Fahrenheit (isFahrenheit = true)
    float f = dht.readTemperature(true);

    // Check if any reads failed and exit early (to try again).
    if (isnan(h) || isnan(t) || isnan(f)) {
      // Serial.println(F("Failed to read from DHT sensor!"));
      return;
    }

    // Serial.print(F("Humidity: "));
    Serial.print("n1.val=");
    Serial.print(h1);
    Serial.write(0xff);
    Serial.write(0xff);
    Serial.write(0xff);      
    // Serial.print(F("%  Temperature: "));
    Serial.print("x0.val=");
    Serial.print(t1);
    Serial.write(0xff);
    Serial.write(0xff);
    Serial.write(0xff);      
    // Serial.println(F("°C "));
    // Serial.print("page 1");
    // Serial.write(0xff);
    // Serial.write(0xff);
    // Serial.write(0xff);      

    if (i % 20 == 0) { 
      Adafruit_MQTT_Publish CO2 = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/CO2");
      Adafruit_MQTT_Publish PM = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/PM");
      Adafruit_MQTT_Publish Humidity = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/Humidity");
      Adafruit_MQTT_Publish Temperature = Adafruit_MQTT_Publish(&mqtt, AIO_USERNAME "/feeds/Temperature");

      if (!CO2.publish(co2ppm)) {
        Serial.println("Failed to send CO2 data");
      } else {
        Serial.println("Sent CO2 data");
      }
      if (!PM.publish(data.pm10_env)) {
        Serial.println("Failed to send PM data");
      } else {
        Serial.println("Sent PM data");
      }
      if (!Humidity.publish(h1)) {
        Serial.println("Failed to send Humidity data");
      } else {
        Serial.println("Sent Humidity data");
      }
      if (!Temperature.publish(t)) {
        Serial.println("Failed to send Temperature data");
      } else {
        Serial.println("Sent Temperature data");
      }
    }
    i = i + 1;
  }
}
